package repositories

import (
	"context"

	"git.dmitriygnatenko.ru/dima/go-common/db"
	sq "github.com/Masterminds/squirrel"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/models"
)

const tagTableName = "tag"

type TagRepository struct {
	db *db.DB
}

func InitTagRepository(db *db.DB) *TagRepository {
	return &TagRepository{db: db}
}

func (t TagRepository) GetAllUsed(ctx context.Context) ([]models.Tag, error) {
	var res []models.Tag

	query := "SELECT t.id, t.url, t.tag " +
		"FROM " + articleTagTableName + " at, " + tagTableName + " t " +
		"WHERE t.id = at.tag_id AND at.article_id IN " +
		"(SELECT id FROM " + articleTableName + " " + "WHERE is_active = true) " +
		"GROUP BY t.id"

	rows, err := t.db.QueryContext(ctx, query)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	for rows.Next() {
		row := models.Tag{}

		err = rows.Scan(&row.ID, &row.URL, &row.Tag)
		if err != nil {
			return nil, err
		}

		res = append(res, row)
	}

	if err = rows.Err(); err != nil {
		return nil, err
	}

	return res, nil
}

func (t TagRepository) GetByURL(ctx context.Context, url string) (*models.Tag, error) {
	query, args, err := sq.Select("id", "url", "tag").
		From(tagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"url": url}).
		Limit(1).
		ToSql()

	if err != nil {
		return nil, err
	}

	var res models.Tag

	err = t.db.QueryRowContext(ctx, query, args...).
		Scan(&res.ID, &res.URL, &res.Tag)

	if err != nil {
		return nil, err
	}

	return &res, nil
}

func (t TagRepository) GetByID(ctx context.Context, id uint64) (*models.Tag, error) {
	query, args, err := sq.Select("id", "url", "tag").
		From(tagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"id": id}).
		Limit(1).
		ToSql()

	if err != nil {
		return nil, err
	}

	var res models.Tag

	err = t.db.QueryRowContext(ctx, query, args...).
		Scan(&res.ID, &res.URL, &res.Tag)

	if err != nil {
		return nil, err
	}

	return &res, nil
}

func (t TagRepository) GetAll(ctx context.Context) ([]models.Tag, error) {
	var res []models.Tag

	query, args, err := sq.Select("id", "url", "tag").
		From(tagTableName).
		ToSql()

	if err != nil {
		return nil, err
	}

	rows, err := t.db.QueryContext(ctx, query, args...)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	for rows.Next() {
		row := models.Tag{}

		err = rows.Scan(&row.ID, &row.URL, &row.Tag)
		if err != nil {
			return nil, err
		}

		res = append(res, row)
	}

	if err = rows.Err(); err != nil {
		return nil, err
	}

	return res, nil
}

func (t TagRepository) GetByArticleID(ctx context.Context, id uint64) ([]models.Tag, error) {
	var res []models.Tag

	query := "SELECT t.id, t.url, t.tag " +
		"FROM " + articleTagTableName + " at, " + tagTableName + " t " +
		"WHERE t.id = at.tag_id AND at.article_id = $1"

	rows, err := t.db.QueryContext(ctx, query, id)
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	for rows.Next() {
		row := models.Tag{}

		err = rows.Scan(&row.ID, &row.URL, &row.Tag)
		if err != nil {
			return nil, err
		}

		res = append(res, row)
	}

	if err = rows.Err(); err != nil {
		return nil, err
	}

	return res, nil
}

func (t TagRepository) IsUsed(ctx context.Context, id uint64) (bool, error) {
	query, args, err := sq.Select("COUNT(tag_id)").
		From(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"tag_id": id}).
		ToSql()

	if err != nil {
		return false, err
	}

	var count int
	err = t.db.QueryRowContext(ctx, query, args...).Scan(&count)
	if err != nil {
		return false, err
	}

	return count > 0, nil
}

func (t TagRepository) Add(ctx context.Context, req models.Tag) error {
	query, args, err := sq.Insert(tagTableName).
		PlaceholderFormat(sq.Dollar).
		Columns("tag", "url").
		Values(req.Tag, req.URL).
		ToSql()

	if err != nil {
		return err
	}

	_, err = t.db.ExecContext(ctx, query, args...)

	return err
}

func (t TagRepository) Update(ctx context.Context, req models.Tag) error {
	query, args, err := sq.Update(tagTableName).
		PlaceholderFormat(sq.Dollar).
		Set("tag", req.Tag).
		Set("url", req.URL).
		Where(sq.Eq{"id": req.ID}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = t.db.ExecContext(ctx, query, args...)

	return err
}

func (t TagRepository) Delete(ctx context.Context, id uint64) error {
	query, args, err := sq.Delete(tagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"id": id}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = t.db.ExecContext(ctx, query, args...)

	return err
}
