package sp

import (
	"log/slog"

	cacheService "git.dmitriygnatenko.ru/dima/go-common/cache/ttl_memory_cache"
	dbService "git.dmitriygnatenko.ru/dima/go-common/db"
	"git.dmitriygnatenko.ru/dima/go-common/logger"
	smtpService "git.dmitriygnatenko.ru/dima/go-common/smtp"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/repositories"
	authService "git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/auth"
	envService "git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/env"
)

type ServiceProvider struct {
	env                  *envService.Service
	cache                *cacheService.Cache
	auth                 *authService.Service
	articleRepository    *repositories.ArticleRepository
	tagRepository        *repositories.TagRepository
	articleTagRepository *repositories.ArticleTagRepository
	userRepository       *repositories.UserRepository
}

func Init() (*ServiceProvider, error) {
	sp := &ServiceProvider{}

	// Init services
	env, err := envService.Init()
	if err != nil {
		return nil, err
	}
	sp.env = env

	cache := cacheService.NewCache(
		cacheService.NewConfig(
			cacheService.WithExpiration(env.CacheDefaultDuration()),
			cacheService.WithCleanupInterval(env.CacheCleanupInterval()),
		),
	)

	sp.cache = cache

	auth, err := authService.Init(sp.env)
	if err != nil {
		return nil, err
	}
	sp.auth = auth

	db, err := dbService.NewDB(
		dbService.NewConfig(
			dbService.WithDriver(env.DbDriver()),
			dbService.WithHost(env.DbHost()),
			dbService.WithPort(uint16(env.DbPort())),
			dbService.WithUsername(env.DbUser()),
			dbService.WithPassword(env.DbPassword()),
			dbService.WithDatabase(env.DbName()),
			dbService.WithMaxIdleConns(env.DbMaxIdleConns()),
			dbService.WithMaxOpenConns(env.DbMaxOpenConns()),
			dbService.WithMaxIdleConnLifetime(env.DbMaxOpenConnLifetime()),
			dbService.WithMaxOpenConnLifetime(env.DbMaxOpenConnLifetime()),
			dbService.WithSSLMode(env.DbSSLMode()),
		),
	)

	if err != nil {
		return nil, err
	}

	smtp, err := smtpService.NewSMTP(
		smtpService.NewConfig(
			smtpService.WithPassword(env.SmtpPassword()),
			smtpService.WithUsername(env.SmtpUser()),
			smtpService.WithHost(env.SmtpHost()),
			smtpService.WithPort(uint16(env.SmtpPort())),
		),
	)

	// Init repositories
	sp.articleRepository = repositories.InitArticleRepository(db)
	sp.tagRepository = repositories.InitTagRepository(db)
	sp.articleTagRepository = repositories.InitArticleTagRepository(db)
	sp.userRepository = repositories.InitUserRepository(db)

	// Init logger
	loggerOpts := logger.ConfigOptions{
		logger.WithSMTPClient(smtp),
		logger.WithEmailLogEnabled(env.LoggerEmailEnabled()),
		logger.WithEmailSubject(env.LoggerEmailSubject()),
		logger.WithEmailRecipient(env.LoggerEmail()),
		logger.WithEmailLogAddSource(env.LoggerEmailAddSource()),
		logger.WithStdoutLogEnabled(env.LoggerStdoutEnabled()),
		logger.WithStdoutLogAddSource(env.LoggerStdoutAddSource()),
	}

	if len(env.LoggerEmailLevel()) > 0 {
		var level slog.Level
		if err = level.UnmarshalText([]byte(env.LoggerEmailLevel())); err != nil {
			return nil, err
		}

		loggerOpts.Add(logger.WithEmailLogLevel(level))
	}

	if len(env.LoggerStdoutLevel()) > 0 {
		var level slog.Level
		if err = level.UnmarshalText([]byte(env.LoggerStdoutLevel())); err != nil {
			return nil, err
		}

		loggerOpts.Add(logger.WithStdoutLogLevel(level))
	}

	if err = logger.Init(logger.NewConfig(loggerOpts...)); err != nil {
		return nil, err
	}

	return sp, nil
}

func (sp *ServiceProvider) EnvService() *envService.Service {
	return sp.env
}

func (sp *ServiceProvider) CacheService() *cacheService.Cache {
	return sp.cache
}

func (sp *ServiceProvider) AuthService() *authService.Service {
	return sp.auth
}

func (sp *ServiceProvider) ArticleRepository() *repositories.ArticleRepository {
	return sp.articleRepository
}

func (sp *ServiceProvider) TagRepository() *repositories.TagRepository {
	return sp.tagRepository
}

func (sp *ServiceProvider) ArticleTagRepository() *repositories.ArticleTagRepository {
	return sp.articleTagRepository
}

func (sp *ServiceProvider) UserRepository() *repositories.UserRepository {
	return sp.userRepository
}
