package admin

import (
	"strconv"

	"github.com/go-playground/validator/v10"
	"github.com/gofiber/fiber/v2"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/helpers"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/models"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/handler"
)

const errTagExists = "Тег с данным URL уже существует"

func TagHandler(
	tagRepository TagRepository,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		tags, err := tagRepository.GetAll(fctx.Context())
		if err != nil {
			return err
		}

		return fctx.Render("admin/tag", fiber.Map{
			"tags":    mapper.ToTagDTOList(tags),
			"section": "tag",
		}, "admin/_layout")
	}
}

func AddTagHandler(
	tagRepository TagRepository,
	cacheService CacheService,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		var form models.TagForm
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helpers.GetDefaultTranslator(validate)
		if err != nil {
			return err
		}

		if fctx.Method() == fiber.MethodPost {
			if err = fctx.BodyParser(&form); err != nil {
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helpers.FormatValidateErrors(err, trans)
			}

			if res, _ := tagRepository.GetByURL(ctx, form.URL); res != nil {
				validateErrors["TagForm.URL"] = errTagExists
			}

			if len(validateErrors) == 0 {
				err = tagRepository.Add(ctx, mapper.ToTag(form))
				if err != nil {
					return err
				}

				return fctx.Redirect("/admin/tag")
			}
		}

		return fctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   "Добавление тега",
		}, "admin/_layout")
	}
}

func EditTagHandler(
	tagRepository TagRepository,
	cacheService CacheService,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helpers.GetDefaultTranslator(validate)
		if err != nil {
			return err
		}

		ID, err := strconv.Atoi(fctx.Params("id"))
		if err != nil {
			return err
		}

		tag, err := tagRepository.GetByID(ctx, ID)
		if err != nil {
			return err
		}

		form := models.TagForm{
			ID:  ID,
			Tag: tag.Tag,
			URL: tag.URL,
		}

		if fctx.Method() == fiber.MethodPost {
			if err = fctx.BodyParser(&form); err != nil {
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helpers.FormatValidateErrors(err, trans)
			}

			if res, _ := tagRepository.GetByURL(ctx, form.URL); res != nil {
				if res.ID != ID {
					validateErrors["TagForm.URL"] = errTagExists
				}
			}

			if len(validateErrors) == 0 {
				err = tagRepository.Update(ctx, mapper.ToTag(form))
				if err != nil {
					return err
				}

				cacheService.Delete(handler.TagCacheKey + tag.URL)

				return fctx.Redirect("/admin/tag")
			}
		}

		return fctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   "Редактирование тега",
		}, "admin/_layout")
	}
}

func DeleteTagHandler(
	tagRepository TagRepository,
	cacheService CacheService,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		ID, err := strconv.Atoi(fctx.Params("id"))
		if err != nil {
			return err
		}

		tag, err := tagRepository.GetByID(ctx, ID)
		if err != nil {
			return err
		}

		used, err := tagRepository.IsUsed(ctx, ID)
		if err != nil {
			return err
		}

		if fctx.Method() == fiber.MethodPost {
			err = tagRepository.Delete(ctx, ID)
			if err != nil {
				return err
			}

			cacheService.Delete(handler.TagCacheKey + tag.URL)

			return fctx.Redirect("/admin/tag")
		}

		return fctx.Render("admin/tag_delete", fiber.Map{
			"tag":     tag.Tag,
			"used":    used,
			"section": "tag",
		}, "admin/_layout")
	}
}
